/*
 * bloom filter for NIST NSRL
 * (C) August 2006, Simson L. Garfinkel
 * Released into the public domain.
 */

#ifndef NSRL_BLOOM_H
#define NSRL_BLOOM_H

#ifdef __cplusplus
extern "C" {
#endif
#ifdef NEVER_DEFINED
}
#endif

typedef struct  nsrl_bloom_ {
    int hash_bits;		// how many bits in the input hash (128 or 160)
    int bloom_bits;		// number of bits to use for each bloom function
    int k;	        // number of bloom functions to use
    int vector_bytes;		// length of the vector (vector_bytes*8 = 2^bloom_bits)
    int vector_offset;		// offset of vector in file (usually 4096)
    unsigned char *vector;		// the bloom filter
    char *comment;
    unsigned int added_items;		// How many items have been added.
    int fd;				// file descriptor for mapped file
    unsigned int memmapped:1;		// true if we need to unmap; otherwise we free(vector)
} nsrl_bloom;
extern int nsrl_klookup_counters[32];


/* Service functions: */
int nsrl_hex2bin(unsigned char *binbuf,const char *hex);

/** nsrl_bloom_create:
 * Create a nsrl bloom filter for either a 128-bit or a 160-bit hash.
 * If fname is specified, the file is created; otherwise the bloom
 * filter is kept in memory. M is log2(m); k is the number of functions to use.
 */
nsrl_bloom *nsrl_bloom_create(const char *fname,int hash_size, int M, int k, const char *comment);
nsrl_bloom *nsrl_bloom_open(const char *fname,int mode);	// open an existing filter
void nsrl_bloom_add(nsrl_bloom *b,const unsigned char *hash); 
int  nsrl_bloom_query(nsrl_bloom *b,const unsigned char *hash);
int  nsrl_bloom_free(nsrl_bloom *b);	// close the bloom filter


void nsrl_bloom_info(char *buf,int buflen,const nsrl_bloom *b);	
void nsrl_calc_histogram(const nsrl_bloom *b,unsigned int counts[256]);
void nsrl_print_histogram(const nsrl_bloom *b,const unsigned int counts[256]);
void queryfile(nsrl_bloom *b, const char *fname);
void walkfile(nsrl_bloom *b, const char *fname);
const char *nsrl_hexbuf(char *dst,int dst_len,const unsigned char *bin,int bytes,int flag);

#define NSRL_HEXBUF_UPPERCASE 0x01
#define NSRL_HEXBUF_SPACE2    0x02
#define NSRL_HEXBUF_SPACE4    0x04

extern int nsrl_debug;

#ifdef NEVER_DEFINED
{
#endif
#ifdef __cplusplus
}
#endif

#ifdef __cplusplus
class Bloom {
public:
    
};
#endif
#endif
