#include <stdarg.h>

#include "xml.h"

static const char *cstr(const string &str){
    return str.c_str();
}

static string xml_lt("&lt;");
static string xml_gt("&gt;");
static string xml_am("&amp;");
static string xml_ap("&apos;");
static string xml_qu("&quot;");




string xml::xmlescape(const string &xml)
{
    string ret;
    for(string::const_iterator i = xml.begin(); i!=xml.end(); i++){
	switch(*i){
	case '>': ret += xml_lt; break;
	case '<': ret += xml_gt; break;
	case '&': ret += xml_am; break;
	case '\'': ret += xml_ap; break;
	case '"': ret += xml_qu; break;
	default:
	    ret += *i;
	}
    }
    return ret;
}

/**
 * Strip an XML string as necessary for a tag name.
 */

string xml::xmlstrip(const string &xml)
{
    string ret;
    for(string::const_iterator i = xml.begin(); i!=xml.end(); i++){
	if(isprint(*i) && !strchr("<>\r\n&'\"",*i)){
	    ret += isspace(*i) ? '_' : tolower(*i);
	}
    }
    return ret;
}

xml::xml()
{
    char buf[32];
    strcpy(buf,"/tmp/tmpXXXXXX");
    int fd = mkstemp(buf);
    temp = fdopen(fd,"r+");
}

void xml::setoutfile(const string outfile)
{
    this->outfile = outfile;
}

void xml::write()
{
    FILE *out = fopen(outfile.c_str(),"w");
    fputs("<?xml version='1.0' encoding='ISO-8859-1'?>\n",out);
    write_dtd(out);
    fseek(temp,0L,SEEK_SET);
    char buf[65536];
    while(!feof(temp)){
	int r = fread(buf,1,sizeof(buf),temp);
	if(r>0) fwrite(buf,1,r,out);
    }
    fclose(out);
}

void xml::write_dtd(FILE *out)
{
    fprintf(out,"<!DOCTYPE fiwalk [\n");

    for(tagset::const_iterator it = tags.begin(); it != file_tags.end(); it++){
	fprintf(out,"<!ELEMENT %s ANY >\n",(*it).c_str());
    }
    fprintf(out,"<!ATTLIST volume startsector CDATA #IMPLIED>\n");
    fprintf(out,"]>\n");
}

void xml::push(const string &tag,const string &attribute)
{
    tags.insert(tag);
    fprintf(temp,"<%s%s%s>\n",tag.c_str(),attribute.size()>0 ? " " : "",attribute.c_str());
}

void xml::xmlout(const string &tag,const string &value,const string &attribute)
{
    tags.insert(tag);
    push(tag,attribute);
    fputs(value.c_str(),temp);
    pop(tag);
}


void xml::pop(const string &tag)
{
    tags.insert(tag);
    fputc('<',temp);
    fputc('/',temp);
    fputs(tag.c_str(),temp);
    fputc('>',temp);
    fputc('\n',temp);
}

void xml::printf(const char *format,...)
{
    va_list ap;
    va_start(ap, format);
    vfprintf(temp,format,ap);
    va_end(ap);
}
